#include "WLANWatch-Client.h"

extern int	SEQUENCE_NUM_START;
extern int	SEQUENCE_NUM_SIZE;
extern int	SEQUENCE_NUM_REDUNDANCY;
extern int	UDP_PACKET_DATA_SIZE;
extern int	TRAFFIC_COUNT;
extern char	*serverIP;
extern int	SERV_TCP_PORT;
extern char	*ifname;
extern int	statusPipe[];
extern int      hintPipe[];
extern int      oopsPipe[];

extern int	WLANWatchCsockfd;
extern char	sniff_MAC[];

extern void     writeToPipe(int fd, char *str);

static char     mesg[300];
static FILE	*fp;
static char	buf[MAXLINELENGTH];
static char	*tok;

char            TxLog		[50];
char            TxLog_temp	[50];
char		logTag		[35];
/*
 * These are the commands that the client sends.
 * They terminate with a '\n'.
 */ 
char		START_SNIFFING  [20];
char		STOP_SNIFFING   [20];
char            IMPORT_RXLOG    [20];
char            EXIT            [20];

char            RxLog[20];

static int fill_sniff_MAC(void);

int WLANWatchC_Setup(void);

int fill_sniff_MAC()
{
	fp = popen("ifconfig", "r");
	if(fp == NULL){
		strcpy(mesg, "fill_sniff_MAC: failed...\n");
		writeToPipe(oopsPipe[1], mesg);
		strcpy(mesg, "Hmm., What Could Be The Reason?...\n");
		writeToPipe(hintPipe[1], mesg);
		return -1;
	}										                
	while(fgets(buf, sizeof(buf), fp) != NULL){
		/* if ifname exists in this output of ifconfig */
		if(strstr(buf, ifname) != NULL){
			tok = strtok(buf, " ");
			tok = strtok(NULL, " ");
			tok = strtok(NULL, " ");
			tok = strtok(NULL, " ");
			tok = strtok(NULL, " ");
			if(strlen(tok) != 17){
				sprintf(mesg, "Suspicious MAC : %s...\n", tok);
				writeToPipe(oopsPipe[1], mesg);
				sprintf(mesg, "The NIC seems to have gone into RFMON mode...\n");
				writeToPipe(hintPipe[1], mesg);
				pclose(fp);
				return -1;
			}
			/* Dont forget to put the '\n' character at the end */
			sprintf(sniff_MAC, "%s\n", tok);
			pclose(fp);
			return 0;
		}	
	}
	sprintf(mesg, "Unknown ifname : %s...\n", ifname);
	writeToPipe(oopsPipe[1], mesg);
	strcpy(mesg, "Is the 'NIC Setup' properly?...\n");
	writeToPipe(hintPipe[1], mesg);
	pclose(fp);
	return -1;
}

int WLANWatchC_Setup()
{
	struct sockaddr_in	serv_addr;
	char			seq_str[10];	
	/* 
	 * Number of times the sequence number appears in each packet.
	 * It should be atleast 2.
	 * Sequence number appears SEQUENCE_NUM_REDUNDANCY-1 times 
	 * immediately after the UDP header and once at the end, in each packet.
	 */
	if(SEQUENCE_NUM_REDUNDANCY < 2){
		strcpy(mesg, "'Sequence Number : Redundancy' should be greater than 1...\n");
		writeToPipe(oopsPipe[1], mesg);
		strcpy(mesg, "Go Change 'Sequence Number : Redundancy'...\n");
		writeToPipe(hintPipe[1], mesg);
		return -1;
	}
	
	/* 
	 * UDP_PACKET_DATA_SIZE should be >= SEQUENCE_NUM_REDUNDANCY * SEQUENCE_NUM_SIZE 
	 * as the sequence number is stored in each packet SEQUENCE_NUM_REDUNDANCY times.
	 */ 
	if(UDP_PACKET_DATA_SIZE < SEQUENCE_NUM_REDUNDANCY * SEQUENCE_NUM_SIZE){
		strcpy(mesg, "'Packet Data Size' is less than 'Sequence Number : Redundancy' * 'Sequence Number : Size'...\n");
		writeToPipe(oopsPipe[1], mesg);
		strcpy(mesg, "Select a larger 'Packet Data Size'...\n");
		writeToPipe(hintPipe[1], mesg);
		return -1;
	}

	sprintf(seq_str, "%d", SEQUENCE_NUM_START + TRAFFIC_COUNT - 1);
	if(strlen(seq_str) > SEQUENCE_NUM_SIZE){
		strcpy(mesg, "The 'Sequence Number : Size' is not big enough w.r.t 'Sequence Number : Start' & 'Traffic Count'...\n");
		writeToPipe(oopsPipe[1], mesg);
		strcpy(mesg, "Select A Larger 'Sequence Number : Size'...\n");
		writeToPipe(hintPipe[1], mesg);
		return -1;
	}

	fp = popen("date", "r");
	if(fgets(buf, sizeof(buf), fp) != NULL){
		sprintf(mesg, "%s", buf);
		writeToPipe(statusPipe[1], mesg);

		tok = strtok(buf, " ");
		strcpy(logTag, tok);

		tok = strtok(NULL, " ");
		strcat(logTag, tok);

		tok = strtok(NULL, " ");
		strcat(logTag, tok);
		strcat(logTag, "-");

		tok = strtok(NULL, " ");
		strcat(logTag, tok);

		pclose(fp);
		
	}else{
		strcpy(mesg, "popen(date) seems to have failed...\n");
		writeToPipe(oopsPipe[1], mesg);
		pclose(fp);
		return -1;
	}
	
	sprintf(TxLog, "log/Tx-%s.log", logTag);
	strcpy(TxLog_temp, ".TxLog_temp.log");
	if(fill_sniff_MAC() == -1)
		return -1;
	
	strcpy(START_SNIFFING,	"START_SNIFFING\n");
	strcpy(STOP_SNIFFING,	"STOP_SNIFFING\n");
	strcpy(IMPORT_RXLOG,	"IMPORT_RXLOG\n");
	strcpy(EXIT,		"EXIT\n");
	sprintf(RxLog,		"log/Rx-%s.log", logTag);
	
	/* 
	 * Fill in the structure "serv_addr" with the address 
	 * of the server that we want to connect to.
	 */
	bzero((char *) &serv_addr, sizeof(serv_addr));
	serv_addr.sin_family      = AF_INET;
	serv_addr.sin_addr.s_addr = inet_addr(serverIP);
	serv_addr.sin_port        = htons(SERV_TCP_PORT);

	if((WLANWatchCsockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0){
		strcpy(mesg, "socket creation failed...\n");
		writeToPipe(oopsPipe[1], mesg);
		strcpy(mesg, "man socket :-)...\n");
		writeToPipe(hintPipe[1], mesg);
		return -1;
	}

	/* Connect to the server. */
	if(connect(WLANWatchCsockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr))  != 0){
		sprintf(mesg, "Connection to WLANWatchS at port %d failed...\n", SERV_TCP_PORT);
		writeToPipe(oopsPipe[1], mesg);
		sprintf(mesg, "Make sure the WLANWatch Server is running on %s at port %d...\n", serverIP, SERV_TCP_PORT);
		writeToPipe(hintPipe[1], mesg);
	
		close(WLANWatchCsockfd);	
		WLANWatchCsockfd = 0;
		return -1;
	}
	sprintf(mesg, "Connected to WLANWatch Server on %s at port %d...\n", serverIP, SERV_TCP_PORT);
	writeToPipe(statusPipe[1], mesg);
	return 0;
}
