#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "WLANWatch-Client.h"

/*------------------------------*/
/*      WLANWatchC Traffic Parameters */
/*------------------------------*/
extern char		*dst_ip;
extern int		src_port;
extern int		dst_port;
extern int		SEQUENCE_NUM_START;
extern int		SEQUENCE_NUM_SIZE;
extern int		SEQUENCE_NUM_REDUNDANCY;
extern int		UDP_PACKET_DATA_SIZE;
extern int		TRAFFIC_COUNT;
extern int		remote_sniff_channel;
extern int		REPEAT_TEST;

/*------------------------------*/
/*      WLANWatchC Configuration      */
/*------------------------------*/
extern char		*serverIP;
extern int		SERV_TCP_PORT;
extern int		nic;
extern char		*ifname;
extern char		*src_ip;
extern char		*tcpdump_path;

/*------------------------------*/
/*      WLANWatchC Configuration      */
/*------------------------------*/
extern char		*c_ifname;
extern char		*c_ip;
extern char		*c_mask;
extern char		*c_gw;
extern char		*c_driver;
extern char		*c_ssid;

extern char		*p_ifname;
extern char		*p_ip;
extern char		*p_mask;
extern char		*p_gw;
extern char		*p_driver;
extern char		*p_ssid;
/*------------------------------*/

extern GtkWidget	*WLANWatch_Client;
extern int		clientUp;
extern int		statusPipe[];
extern int		hintPipe[];
extern int		oopsPipe[];
extern int		stats_clicked;

extern pthread_mutex_t	mutex;

extern int		WLANWatchCsockfd;

extern void*		on_start(void *arg);
extern void*		on_stats(void *arg);


static gchar		*temp;
static pthread_t	on_stats_thread;
static char		mesg[300];

static void		read_dst_ip(void);
static void		read_src_port(void);
static void		read_dst_port(void);
static void		read_seqno_start(void);
static void		read_seqno_size(void);
static void		read_seqno_redundancy(void);
static void		read_packet_size(void);
static void		read_traffic_count(void);
static void		read_remote_sniff_channel(void);
static void		read_serverIP(void);
static void		read_SERV_TCP_PORT(void);
static void		read_ifname(void);
static void		read_src_ip(void);
static void		read_tcpdump_path(void);

static void		read_c_ifname(void);
static void		read_c_ip(void);
static void		read_c_mask(void);
static void		read_c_gw(void);
static void		read_c_driver(void);
static void		read_c_ssid(void);
static void		read_p_ifname(void);
static void		read_p_ip(void);
static void		read_p_mask(void);
static void		read_p_gw(void);
static void		read_p_driver(void);
static void		read_p_ssid(void);
static void		read_ALL();

pthread_t	on_start_thread;
void 		writeToPipe(int fd, char *str);

static void read_ALL()
{
	read_dst_ip();
	read_src_port();
	read_dst_port();
	read_seqno_start();
	read_seqno_size();
	read_seqno_redundancy();
	read_packet_size();
	read_traffic_count();
	read_remote_sniff_channel();
	read_serverIP();
	read_SERV_TCP_PORT();
	read_ifname();
	read_src_ip();
	read_tcpdump_path();
	read_c_ifname();
	read_c_ip();
	read_c_mask();
	read_c_gw();
	read_c_driver();
	read_c_ssid();
	read_p_ifname();
	read_p_ip();
	read_p_mask();
	read_p_gw();
	read_p_driver();
	read_p_ssid();
}

void writeToPipe(int fd, char *str)
{
	pthread_mutex_lock(&mutex);
	write(fd, str, strlen(str)+1);
	pthread_mutex_unlock(&mutex);
}

static void read_dst_ip(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "dst_ip");
	dst_ip          = gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(dst_ip);
}

static void read_src_port(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "src_port");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	src_port	= atoi(temp);
}

static void read_dst_port(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "dst_port");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	dst_port	= atoi(temp);
}

static void read_seqno_start(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "SEQUENCE_NUM_START");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	SEQUENCE_NUM_START	= atoi(temp);
}

static void read_seqno_size(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "SEQUENCE_NUM_SIZE");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	SEQUENCE_NUM_SIZE	= atoi(temp);
}

static void read_seqno_redundancy(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "SEQUENCE_NUM_REDUNDANCY");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	SEQUENCE_NUM_REDUNDANCY	= atoi(temp);
}

static void read_packet_size(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "UDP_PACKET_DATA_SIZE");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	UDP_PACKET_DATA_SIZE	= atoi(temp);
}

static void read_traffic_count(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "TRAFFIC_COUNT");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	TRAFFIC_COUNT	= atoi(temp);
}

static void read_remote_sniff_channel(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "remote_sniff_channel");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	remote_sniff_channel	= atoi(temp);
}

static void read_serverIP(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry	= lookup_widget(GTK_WIDGET(WLANWatch_Client), "serverIP");
	serverIP	= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(serverIP);
}

static void read_SERV_TCP_PORT(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "SERV_TCP_PORT");
	temp		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(temp);
	SERV_TCP_PORT	= atoi(temp);
}

static void read_ifname(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "ifname");
	ifname          = gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(ifname);
}

static void read_src_ip(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "src_ip");
	src_ip          = gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(src_ip);
}

static void read_tcpdump_path(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry	= lookup_widget(GTK_WIDGET(WLANWatch_Client), "tcpdump_path");
	tcpdump_path	= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(tcpdump_path);
}

static void read_c_ifname(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "c_ifname");
	c_ifname        = gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(c_ifname);
}

static void read_c_ip(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "c_ip");
	c_ip		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(c_ip);
}

static void read_c_mask(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "c_mask");
	c_mask          = gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(c_mask);
}

static void read_c_gw(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "c_gw");
	c_gw		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(c_gw);
}

static void read_c_driver(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "c_driver");
	c_driver	= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(c_driver);
}

static void read_c_ssid(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry	= lookup_widget(GTK_WIDGET(WLANWatch_Client), "c_ssid");
	c_ssid		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(c_ssid);
}

static void read_p_ifname(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "p_ifname");
	p_ifname        = gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(p_ifname);
}

static void read_p_ip(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "p_ip");
	p_ip		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(p_ip);
}

static void read_p_mask(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "p_mask");
	p_mask          = gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(p_mask);
}

static void read_p_gw(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "p_gw");
	p_gw		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(p_gw);
}

static void read_p_driver(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry      = lookup_widget(GTK_WIDGET(WLANWatch_Client), "p_driver");
	p_driver	= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(p_driver);
}

static void read_p_ssid(void)
{
	GtkWidget       *text_entry = NULL;
	text_entry	= lookup_widget(GTK_WIDGET(WLANWatch_Client), "p_ssid");
	p_ssid		= gtk_editable_get_chars(GTK_EDITABLE(text_entry), 0, -1);
	g_strstrip(p_ssid);
}

gboolean
on_WLANWatch_Client_delete_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	on_exit_clicked(NULL, NULL);
	return FALSE;
}

void
on_start_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
	if(clientUp){
		strcpy(mesg, "WLANWatch-Client Already UP...\n");
		writeToPipe(oopsPipe[1], mesg);
		strcpy(mesg, "Hit 'Stop' To End Current Session. Then Hit 'Start'...\n");
		writeToPipe(hintPipe[1], mesg);
		return;
	}

	clientUp = 1;
	read_ALL();
	if(pthread_create(&on_start_thread, NULL, on_start, NULL) != 0){
		strcpy(mesg, "Couldn't create on_start thread...\n");
		writeToPipe(oopsPipe[1], mesg);
		clientUp = 0;
		return;
	}
	stats_clicked = 0;
}

void
on_stats_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
	if(stats_clicked)
		return;
	stats_clicked = 1;

	if(!clientUp){
		strcpy(mesg, "WLANWatch Client : Session Not Running...\n");
		writeToPipe(oopsPipe[1], mesg);
		strcpy(mesg, "Hit 'Start' and go ahead...\n");
		writeToPipe(hintPipe[1], mesg);
		return;	
	}
/*
	if(pthread_join(on_start_thread, NULL)){
		clientUp = 0;
		return;
	}
*/
	if(pthread_create(&on_stats_thread, NULL, on_stats, NULL) != 0){
		strcpy(mesg, "Couldn't create on_stats thread...\n");
		writeToPipe(oopsPipe[1], mesg);
		clientUp = 0;
		stats_clicked = 0;
		return;
	}
//	pthread_detach(on_stats_thread);
}

void
on_stop_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
	if(!clientUp){
		strcpy(mesg, "WLANWatch Client : Session Not Running...\n");
		writeToPipe(oopsPipe[1], mesg);
		strcpy(mesg, "Hit 'Start'...\n");
		writeToPipe(hintPipe[1], mesg);
		return;
	}

	clientUp = 0;

	if(pthread_join(on_start_thread, NULL)){
		return;
	}
	if(WLANWatchCsockfd)
		close(WLANWatchCsockfd);

	strcpy(mesg, "WLANWatch Client : Session Closed...\n\n");
	writeToPipe(statusPipe[1], mesg);

	strcpy(mesg, "Hit 'Start' To Start All Over Again...\n");
	writeToPipe(hintPipe[1], mesg);
}

void
on_exit_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
	FILE *fp;

	fp = fopen(".WLANWatch-client.conf", "w");
	if(fp == NULL){
		gtk_main_quit();
	}else{
		read_ALL();
		
		fprintf(fp, "%s\n", dst_ip);		
		fprintf(fp, "%d\n", src_port);
		fprintf(fp, "%d\n", dst_port);
		fprintf(fp, "%d\n", SEQUENCE_NUM_START);
		fprintf(fp, "%d\n", SEQUENCE_NUM_SIZE);
		fprintf(fp, "%d\n", SEQUENCE_NUM_REDUNDANCY);
		fprintf(fp, "%d\n", UDP_PACKET_DATA_SIZE);
		fprintf(fp, "%d\n", TRAFFIC_COUNT);
		fprintf(fp, "%d\n", remote_sniff_channel);
		fprintf(fp, "%d\n", REPEAT_TEST);
		fprintf(fp, "%s\n", serverIP);
		fprintf(fp, "%d\n", SERV_TCP_PORT);
		fprintf(fp, "%d\n", nic);
		fprintf(fp, "%s\n", ifname);
		fprintf(fp, "%s\n", src_ip);
		fprintf(fp, "%s\n", tcpdump_path);
		fprintf(fp, "%s\n", c_ifname);
		fprintf(fp, "%s\n", c_ip);
		fprintf(fp, "%s\n", c_mask);
		fprintf(fp, "%s\n", c_gw);
		fprintf(fp, "%s\n", c_driver);
		fprintf(fp, "%s\n", c_ssid);
		fprintf(fp, "%s\n", p_ifname);
		fprintf(fp, "%s\n", p_ip);
		fprintf(fp, "%s\n", p_mask);
		fprintf(fp, "%s\n", p_gw);
		fprintf(fp, "%s\n", p_driver);
		fprintf(fp, "%s\n", p_ssid);

		fclose(fp);
	}
	gtk_main_quit();
}

void
on_up_c_clicked				(GtkButton       *button,
                                        gpointer         user_data)
{
	char		command[200];
	FILE		*fp;

	read_ALL();

	sprintf(command, "modprobe %s", c_driver);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "NIC Setup : Could not modprobe the driver module : %s\n", c_driver);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "ifconfig %s %s netmask %s", c_ifname, c_ip, c_mask);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "ifconfig %s mtu 2292", c_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "route add default gw %s netmask %s dev %s", c_gw, c_mask, c_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);
}

void
on_down_c_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	char		command[200];
	FILE		*fp;

	read_ALL();

	sprintf(command, "ifconfig %s down", c_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "NIC Setup : '%s' failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "rmmod -r %s", c_driver);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "NIC Setup : '%s' failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);
}

void
on_up_p_clicked				(GtkButton       *button,
                                        gpointer         user_data)
{
	char		command[200];
	FILE		*fp;

	read_ALL();

	sprintf(command, "modprobe %s", p_driver);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "NIC Setup : Could not modprobe the driver module : %s\n", p_driver);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "wlanctl-ng %s lnxreq_ifstate ifstate=disable", p_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "wlanctl-ng %s lnxreq_ifstate ifstate=enable", p_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "wlanctl-ng %s lnxreq_autojoin ssid=\"%s\" authtype=opensystem", p_ifname, p_ssid);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "ifconfig %s %s netmask %s", p_ifname, p_ip, p_mask);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "ifconfig %s mtu 2292", p_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "route add default gw %s netmask %s dev %s", p_gw, p_mask, p_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "%s : failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);
}

void
on_down_p_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	char		command[200];
	FILE		*fp;

	read_ALL();

	sprintf(command, "wlanctl-ng %s lnxreq_ifstate ifstate=disable", p_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "NIC Setup : '%s' failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "ifconfig %s down", p_ifname);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "NIC Setup : '%s' failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

	sprintf(command, "rmmod -r %s", p_driver);
	if( (fp=popen(command, "r")) == NULL){
		sprintf(mesg, "NIC Setup : '%s' failed...\n", command);
		writeToPipe(oopsPipe[1], mesg);
	}
	pclose(fp);

}

void
on_c_close_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	on_exit_clicked(NULL, NULL);
}

void
on_p_close_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	on_exit_clicked(NULL, NULL);
}

