#include "WLANWatch-Client.h"

extern int	SEQUENCE_NUM_START;
extern int	SEQUENCE_NUM_SIZE;
extern int	UDP_PACKET_DATA_SIZE;
extern int	TRAFFIC_COUNT;
extern int      statusPipe[];
extern int      hintPipe[];
extern int      oopsPipe[];

extern void     writeToPipe(int fd, char *str);

extern char	RxLog[];
extern char	TxLog[];
extern char	logTag[];

static int	TxPacketLoss;
static FILE	*fp;
static char	StatsLog[50];
static char     mesg[300];
static long	start_time_sec;
static long	start_time_usec;
static long	stop_time_sec;
static long	stop_time_usec;
static long	timeinsec;
static long	timeinusec;
static double	totaltime;
static long	application_bytes;
static double	THROUGHPUT;

static	int	totalRxPackets;
static	int	errorCount;
static	int	duplicates;
static void	removeSpaces(char *);
static int	countBytesIn(char *);
static int	countPacketsLost(void);
static int	countTxRxPackets(void);
static int	throughput(void);

int		process_TxRxLogs(void);

static int throughput()
{
	FILE	*RxLogfp;
	char	*p;
	
	char	buf	[MAXLINELENGTH];

	if( (RxLogfp = fopen(RxLog, "r")) == NULL){
		sprintf(mesg, "countTxRxPackets: Couldnt open <%s> for processing...\n", RxLog);
		writeToPipe(oopsPipe[1], mesg);
		return -1;
	}
	if(fgets(buf, sizeof(buf), RxLogfp) != NULL){
		start_time_sec = atol(buf);
//		printf("%ld\n", start_time_sec);

		p = &buf[0];
		while(*p != ' ')
			p++;
		p++;
		start_time_usec = atol(p);
//		printf("%ld\n", start_time_usec);
	}
	while(fgets(buf, sizeof(buf), RxLogfp) != NULL)
		;	
	stop_time_sec = atol(buf);
//	printf("%ld\n", stop_time_sec);

	p = &buf[0];
	while(*p != ' ')
		p++;
	p++;
	stop_time_usec = atol(p);
//	printf("%ld\n", stop_time_usec);

	timeinsec = stop_time_sec - start_time_sec;
	timeinusec = stop_time_usec - start_time_usec;
//	printf("timeinusec = %ld\n", timeinusec);
	if(timeinusec >= 0){
		totaltime = timeinsec + timeinusec/1000000.0;
//		printf("Time Taken = %f\n", totaltime);
	}else{
		totaltime = timeinsec - 1 + (1000000+timeinusec)/1000000.0;
//		printf("Time Taken = %f\n", totaltime);
	}

	/* The 3 in the equation below corresponds to the three TCP frames sent for "STOP_SNIFFING" */	
	application_bytes = (totalRxPackets - errorCount - duplicates - 3) * UDP_PACKET_DATA_SIZE;

	THROUGHPUT = (application_bytes * 8)/ totaltime;
	
	sprintf(mesg, "%s : THROUGHPUT                       : %14.3f Kbps\n", RxLog, THROUGHPUT/1000);
	writeToPipe(statusPipe[1], mesg);
	fprintf(fp, "RxLog: THROUGHPUT		: %13.3f Kbps\n", THROUGHPUT/1000);
	
	return 0;
}

/* Removes spaces in the string str & writes back to str */
static void removeSpaces(char *str)
{
	int	i, j;
	char	temp[MAXLINELENGTH];
	
	temp[0] = 0;
	for(i=0, j=0; i<strlen(str); i++){
		if(str[i] == ' ')
			continue;
		temp[j] = str[i];
		j++;
	}
	temp[j] = 0;
	strcpy(str, temp);
}

/* Count the number of bytes in the string str */
static int countBytesIn(char *str)
{
	char	temp[MAXLINELENGTH];

	strcpy(temp, str);
	removeSpaces(temp);
	return strlen(temp)/2;
}

/* Count the number of packets lost */
static int countPacketsLost()
{
	FILE	*RxLogfp;
	int	i, num, size;
	int	packetsLost; 
	int	seqno, seqno_expected;
	
	char	buf	[MAXLINELENGTH];
	char	str	[MAXLINELENGTH];
	char	seqstr	[MAX_SEQUENCE_NUM_SIZE];

	seqno_expected = SEQUENCE_NUM_START;
	duplicates = 0;
	packetsLost = 0;

	/* RxLog */
	if( (RxLogfp = fopen(RxLog, "r")) == NULL){
		sprintf(mesg, "countTxRxPackets: Couldnt open <%s> for processing...\n", RxLog);
		writeToPipe(oopsPipe[1], mesg);
		
		return -1;
	}
	while(fgets(buf, sizeof(buf), RxLogfp) != NULL){
		if(!strcmp(buf, "*\n")){ /* Skip erroneous packet */
			while(fgets(buf, sizeof(buf), RxLogfp) != NULL){
				if(!strcmp(buf, "\n"))
					break;
			}
			continue;
		}
		if(!strcmp(buf, "aaaa 0300 0000 0800\n")){ /* First data line of a non-erroneous packet */
			/* Initialize size to the number of bytes in the first line of packet */
			size = countBytesIn(buf);
			
			/* Skip the second line (16B) of packet as it doesnt contain the sequence number */
			if(fgets(buf, sizeof(buf), RxLogfp) == NULL){
				break;
			}
			/* Add up the number of bytes in the second line to size */
			size += countBytesIn(buf);

			/* The third line (13th byte onwards) of packet contains the sequence number */
			if(fgets(buf, sizeof(buf), RxLogfp) == NULL){
				break;
			}
			str[0] = 0; /* Nullify the string */
			strncat(str, buf, strlen(buf)-1);
			strcat(str, " ");

			/* Add up the number of bytes in the third line to size */
			size += countBytesIn(buf);

			if(UDP_PACKET_DATA_SIZE <= 4){ /* There is no 4th line in this case */
				/* Tiny Packet - Processing Not Done */
				continue;
			}else{
				/* The sequence number might run into the fourth line of packet */
				if(fgets(buf, sizeof(buf), RxLogfp) == NULL){
					break;
				}
				strcat(str, buf);	/* should be enough to extract the seqno */
				removeSpaces(str);	

				/* Add up the number of bytes in the fourth line to size */
				size += countBytesIn(buf);
				/* 
				 * Now, after this, str is ready to extract the seqno from.   
				 * But before going ahead, just make sure that this packet
				 * is indeed the one (logged) transmitted by our card by
				 * comparing the size. For doing it, add to size the number of bytes
				 * in the rest of this packet - until u encounter a new line in the log.
				 */
                                while(fgets(buf, sizeof(buf), RxLogfp) != NULL){
					if(!strcmp(buf, "\n"))
						break;
					/* Add up the bytes until a newline or EOF occurs */
					size += countBytesIn(buf);
				}

				/* 
				 * See if the size we obtained is equal to 36 + UDP_PACKET_DATA_SIZE 
				 * LLC - 8B, IP Header - 20B, UDP Header - 8B
				 */
				if(size != UDP_PACKET_DATA_SIZE + 36) /* This is a rogue packet */
					continue;

				/* Almost sure that this would have been logged by us */
				seqstr[0] = 0; /* Nullify the sequence string */
				for(i=0; i<SEQUENCE_NUM_SIZE; i++){
					sscanf(&str[24+2*i], "%2X", &num);
					seqstr[i] = (char) num;
				}
				seqstr[i] = 0;
				seqno = atoi(seqstr);

				if(seqno >= SEQUENCE_NUM_START && seqno < seqno_expected){
					duplicates++;	
					sprintf(mesg, "%s : DUPLICATE PACKET :%10d\n", RxLog, seqno);
					writeToPipe(statusPipe[1], mesg);
					fprintf(fp, "RxLog: DUPLICATE PACKET :%10d\n", seqno);
//					printf("Duplicate Packet : %d\n", seqno);
					continue;
				}
				if(seqno == seqno_expected){
					seqno_expected++;
					continue;
				}
				if(seqno > seqno_expected){
					packetsLost = packetsLost + seqno - seqno_expected;
					seqno_expected = seqno + 1;
					continue;
				}
			}
		}
	}

	packetsLost = packetsLost + (SEQUENCE_NUM_START + TRAFFIC_COUNT - seqno_expected) - TxPacketLoss;

	sprintf(mesg, "%s : DUPLICATES FOUND             : %10d\n", RxLog, duplicates);
	writeToPipe(statusPipe[1], mesg);
	fprintf(fp, "RxLog: DUPLICATES FOUND		:%10d\n", duplicates);
	
	sprintf(mesg, "%s : PACKET LOSS in Rx               : %10d\n", RxLog, packetsLost);
	writeToPipe(statusPipe[1], mesg);
	fprintf(fp, "RxLog: PACKET LOSS in Rx	:%10d\n", packetsLost);
	
	return 0;
}

/* Count Number of Tx, Rx and Corrupted packets */
static int countTxRxPackets()
{
	FILE	*TxLogfp;
	FILE	*RxLogfp;
	int	totalTxPackets = 0;
	char	buf[MAXLINELENGTH];

	totalRxPackets = 0;
	errorCount = 0;

	/* RxLog */
	if( (RxLogfp = fopen(RxLog, "r")) == NULL){
		sprintf(mesg, "countTxRxPackets: Couldnt open <%s> for processing...\n", RxLog);
		writeToPipe(oopsPipe[1], mesg);
		return -1;
	}
	while(fgets(buf, sizeof(buf), RxLogfp) != NULL){
		if(!strcmp(buf, "*\n")){
			errorCount++;
			continue;
		}
		if(!strcmp(buf, "\n")){
			totalRxPackets++;
		}
	}

	/* TxLog */
	if( (TxLogfp = fopen(TxLog, "r")) == NULL){
		sprintf(mesg, "countTxRxPackets: Couldnt open <%s> for processing...\n", TxLog);
		writeToPipe(oopsPipe[1], mesg);
		return -1;
	}
	while(fgets(buf, sizeof(buf), TxLogfp) != NULL){
		if(!strcmp(buf, "\n")){
			totalTxPackets++;
		}
	}

	TxPacketLoss = 0;
	if(totalTxPackets < TRAFFIC_COUNT){
		TxPacketLoss =  TRAFFIC_COUNT - totalTxPackets;
	}

	sprintf(mesg, "%s : SPECIFIED TRAFFIC COUNT   : %10d\n", TxLog, TRAFFIC_COUNT);
	writeToPipe(statusPipe[1], mesg);
	fprintf(fp, "TxLog: SPECIFIED TRAFFIC COUNT	:%10d\n", TRAFFIC_COUNT);
	
	sprintf(mesg, "%s : ACTUAL PACKETS DUMPED   : %10d\n", TxLog, totalTxPackets);
	writeToPipe(statusPipe[1], mesg);
	fprintf(fp, "TxLog: ACTUAL PACKETS DUMPED	:%10d\n", totalTxPackets);
	
	sprintf(mesg, "%s : PACKET LOSS in Tx               : %10d\n\n", TxLog, TxPacketLoss);
	writeToPipe(statusPipe[1], mesg);
	fprintf(fp, "TxLog: PACKET LOSS in Tx	:%10d\n\n", TxPacketLoss);
	
	sprintf(mesg, "%s : TOTAL PACKETS SNIFFED     : %10d\n", RxLog, totalRxPackets);
	writeToPipe(statusPipe[1], mesg);
	fprintf(fp, "RxLog: TOTAL PACKETS SNIFFED	:%10d\n", totalRxPackets);
	
	sprintf(mesg, "%s : CORRUPTED PACKETS          : %10d\n", RxLog, errorCount);
	writeToPipe(statusPipe[1], mesg);
	fprintf(fp, "RxLog: CORRUPTED PACKETS	:%10d\n", errorCount);

	return 0;
}

int process_TxRxLogs()
{
	sprintf(mesg, "\nSTATISTICS");
	writeToPipe(statusPipe[1], mesg);
	sprintf(mesg, "\n--------------\n");
	writeToPipe(statusPipe[1], mesg);

	sprintf(StatsLog, "log/St-%s.log", logTag);
	fp = fopen(StatsLog, "w+");

	if(countTxRxPackets() == -1){
		fclose(fp);
		return -1;
	}

	if(countPacketsLost() == -1){
		fclose(fp);
		return -1;
	}

	if(throughput() == -1){
		fclose(fp);
		return -1;
	}

	fclose(fp);
	return 0;	
}
